/****************************************************************************

    PROGRAM: DLLVoice.c

    PURPOSE: DLLVoice template for Sound Blaster applications

    FUNCTIONS:

    WinMain () - calls initialization function, processes message loop
    InitApplication () - initializes window data and registers window
    InitInstance () - saves instance handle and creates main window
    MainWndProc () - processes messages
    About () - processes messages for "About" dialog box

    COMMENTS:

        Windows can have several copies of your application running at the
        same time. The variable hInst keeps track of which instance this
        application is so that processing will be to the correct window.

****************************************************************************/

#include "windows.h"        /* required for all Windows applications */
#include "dllvoice.h"       /* specific to this program              */
#include <sndblst.h>        /* required for all Sound Blaster applications */

#define BUFSIZE 256000

WORD    wDLLVersion;        /* verion number of DLL                  */
WORD    wSoundBlasterMsg;   /* Sound Blaster message                 */
LPSTR   lpCardName;         /* name of Sound Blaster card            */
HANDLE  hVoiceMem;          /* handle of allocated memory            */
LPSTR   lpBuffer;           /* buffer to hold voice data             */
HANDLE  hInst;              /* current instance                      */
HWND    hWnd;               /* main window handle.                   */
WORD    wRecordStatus=0;    /* record status                         */
WORD    wPlayStatus=0;      /* play status                           */

/****************************************************************************

    FUNCTION: WinMain (HANDLE, HANDLE, LPSTR, int)

    PURPOSE: calls initialization function, processes message loop

    COMMENTS:

        Windows recognizes this function by name as the initial entry point 
        for the program.  This function calls the application initialization 
        routine, if no other instance of the program is running, and always 
        calls the instance initialization routine.  It then executes a message 
        retrieval and dispatch loop that is the top-level control structure 
        for the remainder of execution. The loop is terminated when a WM_QUIT
        message is received, at which time this function exits the application 
        instance by returning the value passed by PostQuitMessage(). 

        If this function must abort before entering the message loop, it 
        returns the conventional value NULL.  

****************************************************************************/

int PASCAL WinMain (hInstance, hPrevInstance, lpCmdLine, nCmdShow)
HANDLE hInstance;                /* current instance             */
HANDLE hPrevInstance;            /* previous instance            */
LPSTR lpCmdLine;                 /* command line                 */
int nCmdShow;				     /* show-window type (open/icon) */
{
    MSG msg;                     /* message                      */

    if (!hPrevInstance)               /* Other instances of app running?   */
    if (!InitApplication (hInstance)) /* Initialize shared things          */
        return (FALSE);               /* Exits if unable to initialize     */

    /* Perform initializations that apply to a specific instance */

    if (!InitInstance (hInstance, nCmdShow))
        return (FALSE);

    if ((wDLLVersion = sbcGetDLLVersion ()) == NULL)
        return (FALSE);

    if ((lpCardName = sbcGetCardName ()) == NULL)
        return (FALSE);

    wSoundBlasterMsg = RegisterWindowMessage ((LPSTR) "SoundBlaster");

    /* Acquire and dispatch messages until a WM_QUIT message is received. */

    while (GetMessage (&msg,     /* message structure                      */
        NULL,                    /* handle of window receiving the message */
        NULL,                    /* lowest message to examine              */
        NULL))                   /* highest message to examine             */
	{
        TranslateMessage (&msg); /* Translates virtual key codes           */
        DispatchMessage (&msg);  /* Dispatches message to window           */
    }

    sbcTerminateDLL ();

    return (msg.wParam);        /* Returns the value from PostQuitMessage */
}

/****************************************************************************

    FUNCTION: InitApplication (HANDLE)

    PURPOSE: Initializes window data and registers window class

    COMMENTS:

        This function is called at initialization time only if no other 
        instances of the application are running.  This function performs 
        initialization tasks that can be done once for any number of running 
        instances.  

        In this case, we initialize a window class by filling out a data 
        structure of type WNDCLASS and calling the Windows RegisterClass() 
        function.  Since all instances of this application use the same window 
        class, we only need to do this when the first instance is initialized.  

****************************************************************************/

BOOL InitApplication (hInstance)
HANDLE hInstance;                       /* current instance                   */
{
    WNDCLASS  wc;

    /* Fill in window class structure with parameters that describe the       */
    /* main window.                                                           */

    wc.style         = NULL;             /* Class style(s).                    */
    wc.lpfnWndProc   = MainWndProc;      /* Function to retrieve messages for  */
                                         /* windows of this class.             */
    wc.cbClsExtra    = 0;                /* No per-class extra data.           */
    wc.cbWndExtra    = 0;                /* No per-window extra data.          */
    wc.hInstance     = hInstance;        /* Application that owns the class.   */
    wc.hIcon         = LoadIcon (hInstance, "dllvoice");
    wc.hCursor       = LoadCursor (NULL, IDC_ARROW);
    wc.hbrBackground = GetStockObject(WHITE_BRUSH); 
    wc.lpszMenuName  = "DLLVoiceMenu";   /* Name of menu resource in .RC file. */
    wc.lpszClassName = "DLLVoiceWClass"; /* Name used in call to CreateWindow. */

    /* Register the window class and return success/failure code. */

    return (RegisterClass (&wc));

}

/****************************************************************************

    FUNCTION:  InitInstance (HANDLE, int)

    PURPOSE:  Saves instance handle and creates main window

    COMMENTS:

        This function is called at initialization time for every instance of 
        this application.  This function performs initialization tasks that 
        cannot be shared by multiple instances.  

        In this case, we save the instance handle in a static variable and 
        create and display the main program window.  
        
****************************************************************************/

BOOL InitInstance (hInstance, nCmdShow)
    HANDLE          hInstance;          /* Current instance identifier.       */
    int             nCmdShow;           /* Param for first ShowWindow() call. */
{
    /* Save the instance handle in static variable, which will be used in  */
    /* many subsequence calls from this application to Windows.            */

    hInst = hInstance;

    /* Create a main window for this application instance.  */

    hWnd = CreateWindow
    (
        "DLLVoiceWClass",               /* See RegisterClass() call.          */
        "DLLVOICE Sample Application",  /* Text for window title bar.         */
        WS_OVERLAPPEDWINDOW,            /* Window style.                      */
        CW_USEDEFAULT,                  /* Default horizontal position.       */
        CW_USEDEFAULT,                  /* Default vertical position.         */
        CW_USEDEFAULT,                  /* Default width.                     */
        CW_USEDEFAULT,                  /* Default height.                    */
        NULL,                           /* Overlapped windows have no parent. */
        NULL,                           /* Use the window class menu.         */
        hInstance,                      /* This instance owns this window.    */
        NULL                            /* Pointer not needed.                */
    );

    /* If window could not be created, return "failure" */

    if (!hWnd)
        return (FALSE);

    /* Make the window visible; update its client area; and return "success" */

    ShowWindow (hWnd, nCmdShow);   /* Show the window                        */
    UpdateWindow (hWnd);           /* Sends WM_PAINT message                 */
    return (TRUE);                 /* Returns the value from PostQuitMessage */

}

/****************************************************************************

    FUNCTION: MainWndProc (HWND, unsigned, WORD, LONG)

    PURPOSE:  Processes messages

    MESSAGES:

	WM_COMMAND    - application menu (About dialog box)
	WM_DESTROY    - destroy window

    COMMENTS:

	To process the IDM_ABOUT message, call MakeProcInstance() to get the
	current instance address of the About() function.  Then call Dialog
	box which will create the box according to the information in your
    dllvoice.rc file and turn control over to the About() function.  When
	it returns, free the intance address.

****************************************************************************/

long FAR PASCAL MainWndProc (hWnd, message, wParam, lParam)
HWND hWnd;                    /* window handle                   */
unsigned message;             /* type of message                 */
WORD wParam;                  /* additional information          */
LONG lParam;                  /* additional information          */
{
    FARPROC lpProcAbout;      /* pointer to the "About" function */
    WORD wReturnValue;
    PAINTSTRUCT ps;

    switch (message)
    {
        case WM_PAINT:
            BeginPaint (hWnd, &ps);
            EndPaint (hWnd, &ps);
            DisplayRecordFlag (wRecordStatus);
            DisplayPlayFlag (wPlayStatus);
            break;
        case WM_COMMAND:       /* message: command from application menu */
            switch (wParam)
            {
                case IDM_ABOUT:
                    lpProcAbout = MakeProcInstance (About, hInst);

                    DialogBox(hInst,         /* current instance         */
                        "AboutBox",          /* resource to use          */
                        hWnd,                /* parent handle            */
                        lpProcAbout);        /* About() instance address */

                    FreeProcInstance (lpProcAbout);
                    break;
                case IDM_START:
                    StartVoiceProcess ();
                    break;
                case IDM_STOP:
                    StopVoiceProcess ();
                    break;
                case IDM_PAUSE:
                    PauseVoiceProcess ();
                    break;
                case IDM_CONTINUE:
                    ContinueVoiceProcess ();
                    break;
                default:
                    return (DefWindowProc (hWnd, message, wParam, lParam));
            }
            break;

        case WM_DESTROY:          /* message: window being destroyed */
            PostQuitMessage(0);
            break;

        default:
            if (message == wSoundBlasterMsg)
            {
                switch (wParam)
                {
                    case SBM_CALLBACK:
                        sbcCallBack (lParam);
                        break;
                    case SBM_VOC_FLAG:
                        switch (HIWORD (lParam))
                        {
                            case VP_RECORD_MEMUNFMT:
                                wRecordStatus = LOWORD (lParam);
                                UpdateMenu ();
                                DisplayRecordFlag (wRecordStatus);
                                if (LOWORD (lParam) == 0)
                                {
                                    vocSetDacSpeaker (1);
                                    wReturnValue = vocPlayMemUnFormat (lpBuffer, 11000,
                                                    vocGetRecordCount ());
                                    if (wReturnValue)
                                    {
                                        vocCloseDevice ();
                                        ReleaseMemory ();
                                        DisplayErrorCode (wReturnValue);
                                    }
                                }
                                break;
                            case VP_PLAY_MEMUNFMT:
                                wPlayStatus = LOWORD (lParam);
                                UpdateMenu ();
                                DisplayPlayFlag (wPlayStatus);
                                if (LOWORD (lParam) == 0)
                                {
                                    vocCloseDevice ();
                                    ReleaseMemory ();
                                }
                                break;
                        }
                        break;
                }
            }
            else                  /* Passes it on if unproccessed    */
            {
                return (DefWindowProc(hWnd, message, wParam, lParam));
            }
    }
    return (NULL);
}

/****************************************************************************

    FUNCTION: ReleaseMemory ()

    PURPOSE: Return allocated memory block to windows

****************************************************************************/

void ReleaseMemory ()
{
    GlobalPageUnlock (HIWORD (lpBuffer));
    GlobalUnWire (hVoiceMem);
    GlobalFree (hVoiceMem);
}

/****************************************************************************

    FUNCTION: StartVoiceProcess ()

    PURPOSE: Start voice recording

****************************************************************************/

void StartVoiceProcess ()
{
    WORD wReturnCode;

    if (wReturnCode = vocOpenDevice (hWnd))
        DisplayErrorCode (wReturnCode);
    else
    {
        if ((hVoiceMem = GlobalAlloc (GMEM_MOVEABLE, BUFSIZE)) != NULL)
        {
            lpBuffer = GlobalWire (hVoiceMem);
            GlobalPageLock (HIWORD (lpBuffer));
            vocSetDacSpeaker (0);
            if (wReturnCode = vocRecordMemUnFormat (lpBuffer, 11000, BUFSIZE))
            {
                vocCloseDevice ();
                ReleaseMemory ();
                DisplayErrorCode (wReturnCode);
            }
        }
    }
}

/****************************************************************************

    FUNCTION: StopVoiceProcess ()

    PURPOSE: Stop current active voice process

****************************************************************************/

void StopVoiceProcess ()
{
    WORD wReturnValue;

    if (wReturnValue = vocStopVoice ())
        DisplayErrorCode (wReturnValue);
}

/****************************************************************************

    FUNCTION: PauseVoiceProcess ()

    PURPOSE: Pause current active voice process

****************************************************************************/

void PauseVoiceProcess ()
{
    WORD wReturnValue;

    if (wReturnValue = vocPauseVoice ())
        DisplayErrorCode (wReturnValue);
}

/****************************************************************************

    FUNCTION: ContinueVoiceProcess ()

    PURPOSE: Continue previously paused voice process

****************************************************************************/

void ContinueVoiceProcess ()
{
    WORD wReturnValue;

    if (wReturnValue = vocContinueVoice ())
        DisplayErrorCode (wReturnValue);
}

/****************************************************************************

    FUNCTION: DisplayErrorCode (WORD)

    PURPOSE: Display error code

****************************************************************************/

void DisplayErrorCode (error)
WORD error;
{
    char szTmpBuf[20];

    wsprintf (szTmpBuf, "Error code = %d", error);
    MessageBeep (0);
    MessageBox (NULL, szTmpBuf, "Error...", MB_ICONINFORMATION | MB_OK);
}

/****************************************************************************

    FUNCTION: UpdateMenu (void)

    PURPOSE: Update status of menu items

****************************************************************************/

void UpdateMenu ()
{
    if (wRecordStatus)
    {
            EnableMenuItem (GetMenu (hWnd), IDM_START, MF_DISABLED | MF_GRAYED);
            EnableMenuItem (GetMenu (hWnd), IDM_STOP, MF_ENABLED);
    }
    else
    {
        if (wPlayStatus)
        {
            EnableMenuItem (GetMenu (hWnd), IDM_START, MF_DISABLED | MF_GRAYED);
            EnableMenuItem (GetMenu (hWnd), IDM_STOP, MF_DISABLED | MF_GRAYED);
        }
        else
        {
            EnableMenuItem (GetMenu (hWnd), IDM_START, MF_ENABLED);
            EnableMenuItem (GetMenu (hWnd), IDM_STOP, MF_DISABLED | MF_GRAYED);
        }
    }
}

/****************************************************************************

    FUNCTION: DisplayRecordFlag (WORD)

    PURPOSE: Display status of recording process

****************************************************************************/

void DisplayRecordFlag (rec_status)
WORD rec_status;
{
    HDC hDC;
    int x, y;
    char szText[64];

    wsprintf (szText, "Voice Record Flag    = %04x    ", rec_status);
    hDC = GetDC (hWnd);
    x = GetDeviceCaps (hDC, LOGPIXELSX)*1.5;
    y = GetDeviceCaps (hDC, LOGPIXELSY)*0.25;
    TextOut (hDC, x, y, szText, strlen (szText));
    ReleaseDC (hWnd, hDC);
}

/****************************************************************************

    FUNCTION: DisplayPlayFlag (WORD)

    PURPOSE: Display status of playback process

****************************************************************************/

void DisplayPlayFlag (play_status)
WORD play_status;
{
    HDC hDC;
    int x, y;
    char szText[64];

    wsprintf (szText, "Voice Playback Flag = %04x    ", play_status);
    hDC = GetDC (hWnd);
    x = GetDeviceCaps (hDC, LOGPIXELSX)*1.5;
    y = GetDeviceCaps (hDC, LOGPIXELSY)*0.5;
    TextOut (hDC, x, y, szText, strlen (szText));
    ReleaseDC (hWnd, hDC);
}

/****************************************************************************

    FUNCTION: About (HWND, unsigned, WORD, LONG)

    PURPOSE:  Processes messages for "About" dialog box

    MESSAGES:

	WM_INITDIALOG - initialize dialog box
	WM_COMMAND    - Input received

    COMMENTS:

	No initialization is needed for this particular dialog box, but TRUE
	must be returned to Windows.

	Wait for user to click on "Ok" button, then close the dialog box.

****************************************************************************/

BOOL FAR PASCAL About (hDlg, message, wParam, lParam)
HWND hDlg;                                /* window handle of the dialog box */
unsigned message;                         /* type of message                 */
WORD wParam;                              /* message-specific information    */
LONG lParam;
{
    HDC     hDC;
    HDC     hMemoryDC;
    char    szTmpBuf[128];
    BITMAP  Bitmap;
    HBITMAP hBmSBLogo;
    HBITMAP hOldBitmap;
    HANDLE  hMetaFile, hSBlogo;
    PAINTSTRUCT ps;

    switch (message)
    {
        case WM_INITDIALOG:           /* message: initialize dialog box */
            wsprintf (szTmpBuf, "Hardware Card: %s", lpCardName);
            SetDlgItemText (hDlg, IDT_CARDNAME, szTmpBuf);
            wsprintf (szTmpBuf, "DLL Version: %d.%02d", HIBYTE (wDLLVersion),
                                                        LOBYTE (wDLLVersion));
            SetDlgItemText (hDlg, IDT_DLLVERSION, szTmpBuf);
            return (TRUE);

        case WM_COMMAND:                      /* message: received a command */
            if (wParam == IDOK                /* "OK" box selected?          */
                    || wParam == IDCANCEL)    /* System menu close command?  */
            {
                EndDialog (hDlg, TRUE);       /* Exits the dialog box        */
                return (TRUE);
            }
            break;
        case WM_PAINT:
            hBmSBLogo = LoadBitmap (hInst, "sbct");
            GetObject (hBmSBLogo, sizeof (BITMAP), (LPSTR) &Bitmap);
            hDC = GetDC (hDlg);
            hMemoryDC = CreateCompatibleDC (hDC);
            hOldBitmap = SelectObject (hMemoryDC, hBmSBLogo);
            if (hOldBitmap)
            {
                BitBlt (hDC, 3, 3, Bitmap.bmWidth, Bitmap.bmHeight,
                        hMemoryDC, 0, 0, SRCCOPY);
                SelectObject (hMemoryDC, hOldBitmap);
            }
            DeleteDC (hMemoryDC);
            ReleaseDC (hDlg, hDC);
            DeleteObject (hBmSBLogo);
            break;
    }
    return (FALSE);			      /* Didn't process a message    */
}

